// CPU ID for Windows
// By Steven Lawrance

#include <windows.h>

extern void getCPUID(LPTSTR cpu, LPDWORD model, LPDWORD options, LPDWORD extOptions, LPDWORD has3Dnow, LPTSTR longCPU, LPDWORD tlbInfo, LPDWORD L1data, LPDWORD L1inst, LPDWORD L2, LPDWORD ser1, LPDWORD ser2);

LPTSTR getBit(DWORD byte, BYTE bit)
{
  // Return the requested bit as "yes" or "no" (zero-based, little-endian)
  // Use the cheasy backwards/forward method
  return ((byte << (31 - bit)) >> 31)? "yes": "no";
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpszCmdLine, int nCmdShow)
{
  // Get the ID of the CPU
  char string[8192];
  char longCPU[49];
  char cpu[13];
  DWORD next, model, options, hasExtOptions, extOptions, tlbInfo, L1data, L1inst, L2, ser1, ser2;
  ZeroMemory(cpu, 13);
  ZeroMemory(longCPU, 49);
  getCPUID(cpu, &model, &options, &hasExtOptions, &extOptions, longCPU, &tlbInfo, &L1data, &L1inst, &L2, &ser1, &ser2);
  
  // Write the base information and options
  next = wsprintf(string, "CPU Identification: %s, Type %lu, Family %lu, Model %lu, Stepping %lu\n\nCPU Options:\n  Floating point: %s\n  VM86 (VME): %s\n  Debugging extensions: %s\n  Page size extensions (4MB): %s\n  Time stamp counter (TSC): %s\n  Model specific registers: %s\n  PAE (P6 function): %s\n  Machine check exception: %s\n  CMPXCHG8B instruction: %s\n  APIC enabled: %s\n  Memory range registers: %s\n  PTE global bit: %s\n  Machine check architecture: %s\n  Conditional move: %s\n  MMX: %s",
    cpu, model >> 12, (model & 0x0F00) >> 8, (model & 0x00F0) >> 4, model & 0x000F,
    getBit(options, 0), getBit(options, 1), getBit(options, 2), getBit(options, 3), getBit(options, 4), getBit(options, 5), getBit(options, 6), getBit(options, 7), getBit(options, 8), getBit(options, 9), getBit(options, 12), getBit(options, 13), getBit(options, 14), getBit(options, 15), getBit(options, 23));
  
  // If we have extended options, add those too (if Intel, assume no extended options exist)
  if (hasExtOptions && lstrcmp(cpu, "GenuineIntel")) {
    next += wsprintf(&string[next], "\n\nExtended Options:\n  Number available: %lu\n  3DNow: %s\n  Extended MMX (Cyrix): %s", hasExtOptions, getBit(extOptions, 31), getBit(extOptions, 24));
    if (hasExtOptions >= 4)
      next += wsprintf(&string[next], "\n  Full CPU string: %s", longCPU);
    if (hasExtOptions >= 5)
      next += wsprintf(&string[next], "\n  Num I-TLB entries: %lu, Associativity: %lu\n  Num D-TLB entries: %lu, Associativity: %lu\n  L1 data cache size: %lu, Line size: %lu, Lines per tag: %lu, Associativity: %lu\n  L1 instruction cache size: %lu, Line size: %lu, Lines per tag: %lu, Associativity: %lu",
      tlbInfo & 0x00FF, (tlbInfo & 0xFF00) >> 8, (tlbInfo & 0x00FF0000) >> 16, (tlbInfo & 0xFF000000) >> 24, (L1data & 0xFF000000) >> 24, L1data & 0x00FF, (L1data & 0xFF00) >> 8, (L1data & 0x00FF0000) >> 16, (L1inst & 0xFF000000) >> 24, L1inst & 0x00FF, (L1inst & 0xFF00) >> 8, (L1inst & 0x00FF0000) >> 16);
    if (hasExtOptions >= 6)
      next += wsprintf(&string[next], "\n  L2 cache size: %lu, Line size: %lu, Lines per tag: %lu, Associativity: %lu",
      (L2 & 0xFF000000) >> 24, L2 & 0x00FF, (L2 & 0xFF00) >> 8, (L2 & 0x00FF0000) >> 16);
  }
  
  // If we have a serial number, then display that too ;-)
  if (ser1 || ser2)
    wsprintf(&string[next], "\n\nSerial number: %04X-%04X", ser1, ser2);
  
  // Display our results to the user
  MessageBox(HWND_DESKTOP, string, "CPU ID", MB_ICONASTERISK | MB_OK);
  return 0;
}